>{-# OPTIONS -fglasgow-exts #-}

> module KleisliArrow(KleisliArrow, WriterArrow, ExceptArrow, MaybeArrow) where

> import Arrow
> import ArrowTransformer
> import ArrowOps
> import Monad
> import Monoid

> newtype KleisliArrow m a b c = KA (a b (m c))
> unKA (KA x) = x

> rstrength :: Functor f => (f a, b) -> f (a, b)
> rstrength (v, y) = fmap (\x -> (x, y)) v

> instance (Monad m, GenFunctor a m) => Arrow (KleisliArrow m a) where
>	arr f = KA (arr (return . f))
>	KA f >>> KA g = KA (f >>> genmap g >>> arr join)
>	first (KA f) = KA (first f >>> arr rstrength)

> instance (Monad m, GenFunctor a m) =>
>		ArrowTransformer (KleisliArrow m) a where
>	lift f = KA (f >>> arr return)

> instance ArrowFunctor (KleisliArrow m) where
>	arrowmap embed (KA f) = KA (embed f)

The following promotions follow directly from the arrow transformer.

> instance (ArrowReader r a, Monad m, GenFunctor a m) =>
>		ArrowReader r (KleisliArrow m a) where
>       readState = lift readState

> instance (ArrowState s a, Monad m, GenFunctor a m) =>
>		ArrowState s (KleisliArrow m a) where
>       fetch = lift fetch
>       store = lift store

Special cases for the arrows defined here

> instance (ArrowExcept ex a, Monoid w) =>
>		ArrowExcept ex (WriterArrow w a) where
>       raise = lift raise

> instance (ArrowCircuit a, Monoid w) => ArrowCircuit (WriterArrow w a) where
>	delay x = lift (delay x)

> instance (ArrowExcept ex a, ArrowChoice a) =>
>		ArrowExcept ex (MaybeArrow a) where
>       raise = lift raise

> instance (ArrowWriter w a, ArrowChoice a) =>
>		ArrowWriter w (ExceptArrow ex a) where
>       write = lift write

> instance (ArrowWriter w a, ArrowChoice a) =>
>		ArrowWriter w (MaybeArrow a) where
>       write = lift write

liftings of standard classes

> instance (ArrowChoice a, Monad m, GenFunctor a m) =>
>		ArrowChoice (KleisliArrow m a) where
>	left (KA f) = KA (left f >>> arr (liftM Left ||| (return . Right)))

> instance (ArrowApply a, Monad m, GenFunctor a m) =>
>		ArrowApply (KleisliArrow m a) where
>	app = KA (arr (\(KA f, x) -> (f, x)) >>> app)

Take ArrowPlus operations from the monad

> instance (Arrow a, MonadPlus m, GenFunctor a m) =>
>		ArrowZero (KleisliArrow m a) where
>	zeroArrow = KA (arr (const mzero))

> instance (Arrow a, MonadPlus m, GenFunctor a m) =>
>		ArrowPlus (KleisliArrow m a) where
>	KA f <+> KA g = KA (f &&& g >>> arr (uncurry mplus))

promotions

> instance (ArrowRunState s a a', Monad m, GenFunctor a m, GenFunctor a' m) =>
>		ArrowRunState s (KleisliArrow m a) (KleisliArrow m a') where
>	runState (KA f) = KA (runState f >>> arr rstrength)

> instance (ArrowRunReader r a a', Monad m, GenFunctor a m, GenFunctor a' m) =>
>		ArrowRunReader r (KleisliArrow m a) (KleisliArrow m a') where
>	runReader (KA f) = KA (runReader f)

> instance (ArrowNewReader r a, Monad m, GenFunctor a m) =>
>		ArrowNewReader r (KleisliArrow m a) where
>	newReader (KA f) = KA (newReader f)

Specific instances

Writer

> type WriterArrow w = KleisliArrow ((,) w)

> instance Monoid w => Monad ((,) w) where
>	return x = (mempty, x)
>	(w1, x) >>= f = let (w2, y) = f x in (w1 `mappend` w2, y)

> instance (Arrow a, Monoid w) => ArrowWriter w (WriterArrow w a) where
>	write = KA (arr (\x -> (x, ())))

> instance (Arrow a, Monoid w) => ArrowRunWriter w (WriterArrow w a) a where
>	runWriter (KA f) = f >>> arr (\(w, x) -> (x, w))

> instance (Arrow a, Monoid w) => ArrowNewWriter w (WriterArrow w a) where
>	newWriter (KA f) = KA (f >>> arr (\(w, x) -> (w, (x, w))))

> instance (ArrowLoop a, Monoid w) => ArrowLoop (WriterArrow w a) where
>	loop (KA f) = KA (loop (f >>> arr swapenv))
>		where	swapenv ~(w, ~(x, y)) = ((w, x), y)

promotions

> instance (ArrowRunExcept ex a a', Monoid w) =>
>		ArrowRunExcept ex (WriterArrow w a) (WriterArrow w a') where
>	runExcept (KA f) (KA h) = KA (runExcept f h)

> instance (ArrowNewExcept ex a, Monoid w) =>
>		ArrowNewExcept ex (WriterArrow w a) where
>	newExcept (KA f) (KA h) = KA (newExcept f h)

> instance (ArrowRunMaybe a a', Monoid w) =>
>		ArrowRunMaybe (WriterArrow w a) (WriterArrow w a') where
>	runMaybe (KA f) (KA h) = KA (runMaybe f h)

Exceptions

> instance Monad (Either ex) where
>	return = Right
>	Left ex >>= f = Left ex
>	Right y >>= f = f y

> type ExceptArrow ex = KleisliArrow (Either ex)

> instance ArrowChoice a => ArrowExcept ex (ExceptArrow ex a) where
>	raise = KA (arr Left)

Encapsulate an exception-raising computation, by completely handling
the exception.

> instance ArrowChoice a => ArrowRunExcept ex (ExceptArrow ex a) a where
>	runExcept (KA f) h =
>		arr id &&& f >>> arr strength >>> h ||| arr id
>		where	strength (x, Left y) = Left (x, y)
>			strength (x, Right z) = Right z

Handle an exception, with a handler that may raise it again.

> instance ArrowChoice a => ArrowNewExcept ex (ExceptArrow ex a) where
>	newExcept (KA f) (KA h) =
>		KA (arr id &&& f >>> arr strength >>> h ||| arr Right)
>		where	strength (x, Left y) = Left (x, y)
>			strength (x, Right z) = Right z

promotions

> instance (ArrowRunWriter w a a', ArrowChoice a, ArrowChoice a') =>
>		ArrowRunWriter w (ExceptArrow ex a) (ExceptArrow ex a') where
>	runWriter (KA f) = KA (runWriter f >>> arr rstrength)

> instance (ArrowNewWriter w a, ArrowChoice a) =>
>		ArrowNewWriter w (ExceptArrow ex a) where
>	newWriter (KA f) = KA (newWriter f >>> arr rstrength)

> instance (ArrowRunMaybe a a', ArrowChoice a, ArrowChoice a') =>
>		ArrowRunMaybe (ExceptArrow ex a) (ExceptArrow ex a') where
>	runMaybe (KA f) (KA h) = KA (runMaybe f h)

> instance (ArrowChoice a, ArrowLoop a) => ArrowLoop (ExceptArrow ex a) where
>	loop (KA f) = KA (loop (f >>> arr dist))
>		where	dist x = (dist1 x, dist2 x)
>			dist1 (Left e) = Left e
>			dist1 (Right (x, _)) = Right x
>			dist2 (Left e) = undefined
>			dist2 (Right (_, y)) = y

Maybe (a special case of exceptions)

> type MaybeArrow = KleisliArrow Maybe

> maybeToEither :: Maybe a -> Either () a
> maybeToEither Nothing = Left ()
> maybeToEither (Just x) = Right x

> eitherToMaybe :: Either () a -> Maybe a
> eitherToMaybe (Left _) = Nothing
> eitherToMaybe (Right x) = Just x

> instance ArrowChoice a => GenFunctor a Maybe where
>	genmap f = arr maybeToEither >>> right f >>> arr eitherToMaybe

> instance ArrowChoice a => ArrowRunMaybe (MaybeArrow a) a where
>	runMaybe (KA f) h =
>		arr id &&& f >>> arr strength >>> h ||| arr id
>		where	strength (x, Nothing) = Left x
>			strength (x, Just y) = Right y

promotions

> instance (ArrowRunWriter w a a', ArrowChoice a, ArrowChoice a') =>
>		ArrowRunWriter w (MaybeArrow a) (MaybeArrow a') where
>	runWriter (KA f) = KA (runWriter f >>> arr rstrength)

> instance (ArrowNewWriter w a, ArrowChoice a) =>
>		ArrowNewWriter w (MaybeArrow a) where
>	newWriter (KA f) = KA (newWriter f >>> arr rstrength)

> instance (ArrowRunExcept ex a a', ArrowChoice a, ArrowChoice a') =>
>		ArrowRunExcept ex (MaybeArrow a) (MaybeArrow a') where
>	runExcept (KA f) (KA h) = KA (runExcept f h)

> instance (ArrowNewExcept ex a, ArrowChoice a) =>
>		ArrowNewExcept ex (MaybeArrow a) where
>	newExcept (KA f) (KA h) = KA (newExcept f h)
